interface RoninSchemaMetadata {
    createdAt: Date | string;
    updatedAt: Date | string;
    createdBy: RoninSchemaAccount;
    updatedBy: RoninSchemaAccount;
    status: 'draft' | 'published' | 'archived';
    locked: boolean;
}
type BillingPlan = 'free' | 'pro' | 'business' | 'enterprise';
interface Avatar {
    key: string;
    meta: {
        size: number;
        type: string;
        width: number;
        height: number;
    };
    placeholder: {
        base64: string;
    };
    src: string;
}
interface Card {
    brand: string;
    expirationMonth: number;
    expirationYear: number;
    last4: string;
    type: string;
}
interface InviteCodes {
    refreshedAt: Date | string;
    roles: {
        owner: string;
        editor: string;
        viewer: string;
        developer: string;
    };
}
interface Invoice {
    companyName: string;
    recipment: string;
    language: string;
    purchaseOrder: unknown | null;
    streetLineOne: string;
    streetLineTwo: unknown | null;
    postalCode: string;
    city: string;
    region: string;
    taxType: string;
    taxNumber: string;
}
interface RoninSchemaAccount {
    id: string;
    avatar: Avatar;
    email: string;
    emailVerificationSentAt: Date | string;
    emailVerificationToken: string;
    emailVerified: boolean;
    firstName: string;
    guaranteeEmailUpdates: boolean;
    handle: string;
    ignoredEmailUpdates: unknown | null;
    language: unknown | null;
    lastName: string;
    lastUsedSpace: string;
    location: string;
    mondayWeekStart: boolean;
    phoneNumber: unknown | null;
    phoneNumberVerified: boolean;
    spaceEmails: unknown | null;
    supportAccess: boolean;
    twitterHandle: string;
    password: string;
    ronin: RoninSchemaMetadata;
}
interface RoninSchemaSpace {
    avatar: Avatar;
    card: Card;
    customer: string;
    database: string;
    dataLocation: string;
    emailDomain: string;
    forceTwoFactorAuth: boolean;
    handle: string;
    id: string;
    inviteCodes: InviteCodes;
    invoice: Invoice;
    location: string;
    name: string;
    plan: BillingPlan;
    site: string;
    sso: boolean;
    suggested: boolean;
    twitterHandle: string;
    updates: unknown | null;
    usedAt: Date | string;
    ronin: RoninSchemaMetadata;
}

type RecursivePartial<T> = {
    [P in keyof T]?: RecursivePartial<T[P]>;
};
type QueryInstructions = {
    with: Record<string, number | string | null | any>;
    where: any;
    to: Record<string, number | string | null | any>;
    including: string[];
    excluding: string[];
    orderedBy: {
        ascending: string[];
        descending: string[];
    };
    in: string;
    after: number;
    limitedTo: number;
};
type QueryType = 'get' | 'set' | 'drop' | 'create' | 'count';
type QuerySchema = Record<string, QueryInstructions>;
type Query = Record<QueryType, QuerySchema>;
type QueryResultList<T> = T[] & {
    moreAfter?: number;
};
type QueryResult<T> = T extends never ? null | number | unknown | QueryResultList<unknown> : T extends Array<infer T> ? QueryResultList<T> : T;
type ReadableValue = File | ReadableStream;
type Reference = {
    key: string;
} | Media;
interface Readable {
    /** Index and type of the Query the `Readable` was found in. */
    query: {
        index: number;
        type: 'set' | 'create';
    };
    /** Schema to update. */
    schema: string;
    /** Name of the field to set the value to. */
    field: string;
    /** Stream containing the value to be set. */
    value: ReadableValue;
    /** Content-Type of the streamed value. */
    contentType: string;
}
interface StoredReadable extends Readable {
    /** Reference to the stored value or Media object. */
    reference: Reference;
}
type MediaMetadata = {
    /** Size of the image or video in bytes. */
    size: number | null;
    /** MIME type of the image or video. */
    type: string | null;
    /** Width of the image or video in pixels. */
    width: number | null;
    /** Height of the image or video in pixels. */
    height: number | null;
};
type MediaPlaceholder = {
    /** Base64 representation of the generated placeholder. */
    base64: string | null;
};
type StoredMedia = {
    /** Key under which the media item is stored inside RONIN. */
    key: string;
    /** Object containing various metadata of the stored media item. */
    meta: MediaMetadata;
    /** Object containing different versions of the generated placeholder. */
    placeholder: MediaPlaceholder | null;
};
type Media = StoredMedia & {
    /** URL under which the stored image or video can be accessed from the client.  */
    src: string;
};

type Fetcher = typeof fetch;

type ReadableProcessingOptions = Partial<{
    /**
     * Allows for specifying a custom endpoint `Readable`s will be uploaded to.
     *
     * This endpoint has to respond with a JSON object of the following format:
     *
     * ```js
     * const res = (...) => {
     *   // Where the value of `id` has to be a valid UUID v4.
     *   key: "35bc40a1-eaec-4163-8259-37c6b9723740"
     * }
     * ```
     */
    endpoint: string;
    /** Allows for specifying a custom HTTP request method. Defaults to `PUT`. */
    method: string;
    /** Allows for specifying a custom `fetch` function used to make network requests. */
    fetcher?: Fetcher;
}>;
/**
 * ## 🚧 For internal use only! 🚧
 *
 * Store `Readable`s from given queries and populate the queries with the respective references.
 * @param queries `Query` array which may or may not include `Readable`s.
 * @param options `ReadableProcessingOptions` object containing options passed on to the internal handling of `Readable`s.
 * @returns Queries populated with the references to each `Readable` they might have initially contained.
 */
declare const processReadables: (queries: Query[], token: string, options?: ReadableProcessingOptions) => Promise<Query[]>;

interface SubFilter {
    is: string | number;
    isNot: string;
    startsWith: string;
    endsWith: string;
    contains: string;
    isGreaterThan: number;
    isLessThan: number;
}
interface Where {
    id: SubFilter;
    description: SubFilter;
    name: SubFilter;
    slug: SubFilter;
    amount: SubFilter;
}
interface With {
    id: string | number;
    description: string | number;
    name: string | number;
    firstName: string;
    lastName: string;
    slug: string | number;
    pluralSlug: string | number;
    amount: string | number;
    handle: string | number;
    usedAt: string | number;
    createdAt: string | number;
    updatedAt: string | number;
    creationTime: string | number;
    invoice: {
        companyName: string;
    };
    team: string | number;
    avatar: string | ReadableValue;
    space: Partial<RoninSchemaSpace>;
}
type AllFields = 'createdAt' | 'updatedAt' | 'name' | 'slug' | 'amount';
type RecordFields = 'views' | 'accounts' | 'spaces';
interface OrderedBy {
    /** Order the resulting records in descending order using a specific field. */
    descending: AllFields[];
    /** Order the resulting records in ascending order using a specific field. */
    ascending: AllFields[];
}
type To = With;
type Variants = 'english' | 'german' | 'japanese';
interface Filter {
    /** Perform specific matching actions (such as "is", "contains", or "startsWith") on the fields of a record. */
    where: Partial<Where>;
    /** Perform "is" matching actions on the fields of a record. */
    with: Partial<With>;
    /** Order the resulting records using a specific field. */
    orderedBy: Partial<OrderedBy>;
    /** Resolve records in a specific variant. */
    in: Partial<Variants>;
    /** Resolve the records associated with certain fields of type "Record". */
    including: Partial<RecordFields>[];
}
interface FilterAndMutate extends Filter {
    /** Perform mutations on the fields of a record or add new fields to it. */
    to: Partial<To>;
}
interface Getter {
    /** Retrieve a single record of the Account schema. */
    account: Partial<Filter>;
    /** Retrieve multiple records of the Account schema. */
    accounts: Partial<Filter>;
    /** Retrieve a single record of the App schema. */
    app: Partial<Filter>;
    /** Retrieve multiple records of the App schema. */
    apps: Partial<Filter>;
    /** Retrieve a single record of the Member schema. */
    member: Partial<Filter>;
    /** Retrieve multiple records of the Member schema. */
    members: Partial<Filter>;
    /** Retrieve a single record of the Schema schema. */
    schema: Partial<Filter>;
    /** Retrieve multiple records of the Schema schema. */
    schemas: Partial<Filter>;
    /** Retrieve a single record of the Space schema. */
    space: Partial<Filter>;
    /** Retrieve multiple records of the Space schema. */
    spaces: Partial<Filter>;
    /** Retrieve a single record of the Session schema. */
    session: Partial<Filter>;
    /** Retrieve multiple records of the Session schema. */
    sessions: Partial<Filter>;
    /** Retrieve a single record of the Team schema. */
    team: Partial<Filter>;
    /** Retrieve multiple records of the Team schema. */
    teams: Partial<Filter>;
    /** Retrieve a single record of the Variant schema. */
    variant: Partial<Filter>;
    /** Retrieve multiple records of the Variant schema. */
    variants: Partial<Filter>;
    /** Retrieve a single record of the View schema. */
    view: Partial<Filter>;
    /** Retrieve multiple records of the View schema. */
    views: Partial<Filter>;
}
interface Setter {
    /** Modify or create a single record of the Account schema. */
    account: Partial<FilterAndMutate>;
    /** Modify or create multiple records of the Account schema. */
    accounts: Partial<FilterAndMutate>;
    /** Modify or create a single record of the App schema. */
    app: Partial<FilterAndMutate>;
    /** Modify or create multiple records of the App schema. */
    apps: Partial<FilterAndMutate>;
    /** Modify or create a single record of the Member schema. */
    member: Partial<FilterAndMutate>;
    /** Modify or create multiple records of the Member schema. */
    members: Partial<FilterAndMutate>;
    /** Modify or create a single record of the Schema schema. */
    schema: Partial<FilterAndMutate>;
    /** Modify or create multiple records of the Schema schema. */
    schemas: Partial<FilterAndMutate>;
    /** Modify or create a single record of the Space schema. */
    space: Partial<FilterAndMutate>;
    /** Modify or create multiple records of the Space schema. */
    spaces: Partial<FilterAndMutate>;
    /** Modify or create a single record of the Team schema. */
    team: Partial<FilterAndMutate>;
    /** Modify or create multiple records of the Team schema. */
    teams: Partial<FilterAndMutate>;
    /** Modify or create a single record of the Variant schema. */
    variant: Partial<FilterAndMutate>;
    /** Modify or create multiple records of the Variant schema. */
    variants: Partial<FilterAndMutate>;
    /** Modify or create a single record of the View schema. */
    view: Partial<FilterAndMutate>;
    /** Modify or create multiple records of the View schema. */
    views: Partial<FilterAndMutate>;
}
interface Dropper {
    /** Remove a single record of the Account schema. */
    account: Partial<Filter>;
    /** Remove multiple records of the Account schema. */
    accounts: Partial<Filter>;
    /** Remove a single record of the App schema. */
    app: Partial<Filter>;
    /** Remove multiple records of the App schema. */
    apps: Partial<Filter>;
    /** Remove a single record of the Member schema. */
    member: Partial<Filter>;
    /** Remove multiple records of the Member schema. */
    members: Partial<Filter>;
    /** Remove a single record of the Schema schema. */
    schema: Partial<Filter>;
    /** Remove multiple records of the Schema schema. */
    schemas: Partial<Filter>;
    /** Remove a single record of the Space schema. */
    space: Partial<Filter>;
    /** Remove multiple records of the Space schema. */
    spaces: Partial<Filter>;
    /** Remove a single record of the Team schema. */
    team: Partial<Filter>;
    /** Remove multiple records of the Team schema. */
    teams: Partial<Filter>;
    /** Remove a single record of the Variant schema. */
    variant: Partial<Filter>;
    /** Remove multiple records of the Variant schema. */
    variants: Partial<Filter>;
    /** Remove a single record of the View schema. */
    view: Partial<Filter>;
    /** Remove multiple records of the View schema. */
    views: Partial<Filter>;
}
interface Creator {
    /** Create a single record of the Account schema. */
    account: Partial<Filter>;
    /** Create multiple records of the Account schema. */
    accounts: Partial<Filter>;
    /** Create a single record of the App schema. */
    app: Partial<Filter>;
    /** Create multiple records of the App schema. */
    apps: Partial<Filter>;
    /** Create a single record of the Member schema. */
    member: Partial<Filter>;
    /** Create multiple records of the Member schema. */
    members: Partial<Filter>;
    /** Create a single record of the Schema schema. */
    schema: Partial<Filter>;
    /** Create multiple records of the Schema schema. */
    schemas: Partial<Filter>;
    /** Create a single record of the Space schema. */
    space: Partial<Filter>;
    /** Create multiple records of the Space schema. */
    spaces: Partial<Filter>;
    /** Create a single record of the Team schema. */
    team: Partial<Filter>;
    /** Create multiple records of the Team schema. */
    teams: Partial<Filter>;
    /** Create a single record of the Variant schema. */
    variant: Partial<Filter>;
    /** Create multiple records of the Variant schema. */
    variants: Partial<Filter>;
    /** Create a single record of the View schema. */
    view: Partial<Filter>;
    /** Create multiple records of the View schema. */
    views: Partial<Filter>;
    /** Create a single record of the Event schema. */
    event: Partial<Filter>;
    /** Create multiple records of the Event schema. */
    events: Partial<Filter>;
}
interface Counter {
    /** Count records of the Account schema. */
    accounts: Partial<Filter>;
    /** Count records of the App schema. */
    apps: Partial<Filter>;
    /** Count records of the Member schema. */
    members: Partial<Filter>;
    /** Count records of the Schema schema. */
    schemas: Partial<Filter>;
    /** Count records of the Space schema. */
    spaces: Partial<Filter>;
    /** Count records of the Team schema. */
    teams: Partial<Filter>;
    /** Count records of the Variant schema. */
    variants: Partial<Filter>;
    /** Count records of the View schema. */
    views: Partial<Filter>;
}

type QueryProcessingOptions = {
    /**
     * Allows for specifying a custom HTTP endpoint that the provided queries
     * should be submitted to.
     */
    endpoint?: string;
    /**
     * Allows for specifying a custom replacement for the native JavaScript
     * `fetch` function used to make network requests for submitting queries.
     */
    fetcher?: Fetcher;
    /**
     * Allows for specifying a custom source database other than the one
     * associated with the provided app token.
     */
    database?: string;
};

type RunScratchpadOptions = {
    /**
     * Allows for specifying custom data hooks that are executed for every query
     * of a particular schema.
     */
    hooks?: Record<string, any>;
    /**
     * Allows for passing a custom app token. In a Node.js-based environment,
     * `process.env.RONIN_TOKEN` will be used by default.
     */
    token?: string;
    /**
     * ## 🚧 For internal use only! 🚧
     *
     * Allows for specifying custom options used for processing readables.
     */
    readableProcessingOptions?: ReadableProcessingOptions;
    /**
     * ## 🚧 For internal use only! 🚧
     *
     * Allows for specifying custom options used for processing queries.
     */
    queryProcessingOptions?: QueryProcessingOptions;
};
type ScratchPad = (actions: {
    /**
     * Retrieve existing records.
     *
     * Read more at https://docs.ronin.co/queries
     *
     * ```js
     * // Simple Example
     * get.account.with.handle = "juri";
     *
     * // Advanced Example
     * get.accounts = {
     *   where: {
     *     role: {
     *       is: "ADMIN" // 👈🏻 Match specific fields
     *     }
     *   }
     *   orderedBy: {
     *     ascending: ["createdAt"]
     *   },
     *   include: ["roles"], // 👈🏻 Resolve nested records
     * }
     * ```
     */
    get: Partial<Getter>;
    /**
     * Modify existing records.
     *
     * Read more at https://docs.ronin.co/queries
     *
     * ```js
     * // Example
     * set.account = {
     *   with: {
     *     handle: "juri",
     *   },
     *   to: {
     *     verified: true,
     *   },
     * }
     * ```
     */
    set: Partial<Setter>;
    /**
     * Create a new record.
     *
     * Read more at https://docs.ronin.co/queries
     *
     * ```js
     * // Example
     * create.account.with = {
     *   handle: "juri",
     *   verified: false,
     * }
     * ```
     */
    create: Partial<Creator>;
    /**
     * Drop existing records.
     *
     * Read more at https://docs.ronin.co/queries
     *
     * ```js
     * // Example
     * drop.accounts.where.handle.isNot = "juri",
     * ```
     */
    drop: Partial<Dropper>;
    /**
     * Count matching records.
     *
     * Read more at https://docs.ronin.co/queries
     *
     * ```js
     * // Example:
     * count.accounts.where.verified.is = true;
     * ```
     */
    count: Partial<Counter>;
}) => void;

type BeforeHook = (query: RecursivePartial<QueryInstructions>, multipleRecords: boolean) => Promise<RecursivePartial<QueryInstructions>>;
type DuringHook<T = unknown> = (query: RecursivePartial<QueryInstructions>, multipleRecords: boolean) => Promise<T>;
type AfterHook<T = unknown> = (query: RecursivePartial<QueryInstructions>, multipleRecords: boolean, result: QueryResult<T>) => Promise<void>;

/**
 * Transform a given `ScratchPad` into `Queries` (array) which can be passed to RONIN.
 * @param scratchPad `ScratchPad` to get executable `Queries` from.
 * @returns `Queries` which will be passed to RONIN.
 */
declare const getQueriesFromScratchPad: (scratchPad: ScratchPad) => Query[];

declare const runQueries: <T>(queries: Query[], options?: RunScratchpadOptions) => Promise<QueryResult<T>[]>;

type ScratchPadFactory = <T>(scratchPad: ScratchPad) => Promise<QueryResult<T>[]>;
declare function handleScratchPadOrConfig(config?: RunScratchpadOptions): ScratchPadFactory;
declare function handleScratchPadOrConfig<T extends object | Array<unknown> = [unknown]>(scratchPad: ScratchPad, config?: RunScratchpadOptions): Promise<T extends [] ? [] : T extends [infer First, ...infer Rest] ? (Rest extends unknown ? T : Array<First>) : [T]>;

export { AfterHook, BeforeHook, DuringHook, Query, QueryResult, Readable, ReadableValue, RunScratchpadOptions, ScratchPad, StoredReadable, handleScratchPadOrConfig as default, getQueriesFromScratchPad, processReadables, runQueries };
